import os
import asyncio
import argparse
import json
import functools
import operator

from . import helpers as _helpers
from . import enums   as _enums
from . import events  as _events
from . import client  as _client
from . import utils   as _utils
from . import widget  as _widget


__all__ = ('main',)


_parser = argparse.ArgumentParser(
    description = 'manage a discord client'
)

_parser.add_argument(
    '--token',
    help = 'the client\'s type-less authentication token (if missing, sought as DISCORD_TOKEN env-var)',
    default = None
)


def _get_client(info):

    if (token := info.token) is None:
        try:
            token = os.environ['DISCORD_TOKEN']
        except KeyError:
            print('token not specified or set in environment')
            exit()

    token = 'Bot ' + token

    client = _client.Client(token = token)

    return client


_manage_parsers = _parser.add_subparsers(
    dest = 'action',
    required = True
)

def _main(args = None):

    info = _parser.parse_args(args)

    space = globals()

    function = space[f'_main_{info.action}']
    
    function(info)
  

_manage_start_parser = _manage_parsers.add_parser(
    'start',
    description = 'start the client'
)

_manage_start_parser.add_argument(
    'path',
    help = 'the entry widget\'s location'
)

_manage_start_parser.add_argument(
    '--intents',
    help = 'the intents to use (comma-delimited)',
    type = lambda data: functools.reduce(operator.xor, map(_enums.Intents.__getitem__, data.split(','))),
    default = 0,
    required = False
)

_manage_start_parser.add_argument(
    '--shard-ids',
    help = 'the shard ids to use (comma-delimited)',
    type = lambda data: tuple(map(int, data.split(','))),
    default = None,
    required = False
)

_manage_start_parser.add_argument(
    '--shard-count',
    help = 'the shard count',
    type = lambda data: int(data),
    default = None,
    required = False
)

_main_start_enter_default_Events = {
    # required for voice
    _events.UpdateVoiceState
}

async def _main_start_enter(client, path, intents, shard_ids, shard_count):

    path, name = os.path.split(path)

    await _widget.load(client, name, path)

    Events = set(_main_start_enter_default_Events)

    for asset in _widget._assets.values():
        Events.update(asset.Events)

    intents |= _utils.get_eventful_intents(Events)

    await client.start(intents, shard_ids, shard_count)

    await client.ready()

    return name

async def _main_start_leave(client, name):

    await client.stop()

    await _widget.drop(client, name)


def _main_start(info):

    loop = asyncio.get_event_loop()

    client = _get_client(info)

    coro = _main_start_enter(client, info.path, info.intents, info.shard_ids, info.shard_count)
    name = loop.run_until_complete(coro)

    try:
        loop.run_forever()
    except KeyboardInterrupt:
        pass
    finally:
        coro = _main_start_leave(client, name)
        loop.run_until_complete(coro)


_manage_update_parser = _manage_parsers.add_parser(
    'update',
    description = 'update the client\'s commands',
)

_manage_update_parser.add_argument(
    'path',
    help = 'the command file\'s extension-less location'
)

async def _main_update_perform(client: _client.Client, commands):

    application = await client.get_self_application_information()
    
    await client.update_all_global_application_commands(application.id, commands)

    await client.stop()

def _main_update(info):

    client = _get_client(info)

    loop = asyncio.get_event_loop()

    base_path = info.path
    init_path = base_path + '.py'
    data_path = base_path + '.json'

    try:
        module = _helpers.load_module_via_path('commands', init_path, attach = False)
    except ModuleNotFoundError:
        with open(data_path, 'r') as file:
            commands = json.load(file)
    else:
        commands = module.commands
        with open(data_path, 'w') as file:
            json.dump(commands, file, indent = 4)

    coro = _main_update_perform(client, commands)
    loop.run_until_complete(coro)


main = _main




import typing

from . import types   as _types
from . import enums   as _enums
from . import objects as _objects


__all__ = (
    'get_public_archived_threads', 'get_private_archived_threads',
    'get_self_private_archived_threads', 'get_active_guild_threads',
    'get_self_authorization_information'
)


class get_public_archived_threads(typing.NamedTuple):

    threads       : list[_objects.Channel]
    thread_members: list[_objects.ThreadMember]
    outstanding   : _types.Boolean


class get_private_archived_threads(typing.NamedTuple):

    threads       : list[_objects.Channel]
    thread_members: list[_objects.ThreadMember]
    outstanding   : _types.Boolean


class get_self_private_archived_threads(typing.NamedTuple):

    threads       : list[_objects.Channel]
    thread_members: list[_objects.ThreadMember]
    outstanding   : _types.Boolean


class get_active_guild_threads(typing.NamedTuple):

    threads       : list[_objects.Channel]
    thread_members: list[_objects.ThreadMember]


class get_self_authorization_information(typing.NamedTuple):

    application: _objects.Application
    scopes     : list[_types.String]
    expires    : _types.ISO8601Timestamp
    core_user  : _objects.User


import sys
import os
import importlib.util


__all__ = ()


def get_shard_id_via_guild_id(shard_count, guild_id):

    return (int(guild_id) >> 22) % shard_count


def yank_dict(origin, keys):

    target = {}
    for key in keys:
        try:
            value = origin[key]
        except KeyError:
            continue
        target[key] = value

    return target


def load_module_via_path(name, path, attach = True, execute = True):

    if os.path.isdir(path):
        path = os.path.join(path, '__init__.py')

    spec = importlib.util.spec_from_file_location(name, path)

    if spec is None:
        raise ModuleNotFoundError(path)

    module = importlib.util.module_from_spec(spec)

    if attach:
        sys.modules[module.__name__] = module

    if execute:
        spec.loader.exec_module(module)

    return module


def drop_module(source):

    for target in tuple(sys.modules):
        if not source.startswith(target):
            continue
        del sys.modules[target]


import asyncio
import aiocord

async def start(client):
    await client.start()
    await client.ready()
    await aiocord.widget.load(client, 'blep', './modules')

async def stop(client):
    await aiocord.widget.drop(client, 'blep')
    await client.stop()

loop = asyncio.get_event_loop()

client = aiocord.client.Client(token = ...)

loop.run_until_complete(start(client))

try:
    loop.run_forever()
except KeyboardInterrupt:
    pass
finally:
    loop.run_until_complete(stop(client))




from . import client as _client


__all__ = (
    'Route',
    'get_global_application_commands', 'create_global_application_command', 
    'get_global_application_command', 'update_global_application_command', 
    'delete_global_application_command', 
    'update_all_global_application_commands', 'get_guild_application_commands', 
    'create_guild_application_command', 'get_guild_application_command', 
    'update_guild_application_command', 'delete_guild_application_command', 
    'update_all_guild_application_commands', 
    'get_guild_application_command_permissions', 
    'get_application_command_permissions', 
    'update_application_command_permissions', 
    # 'update_all_application_command_permissions', 
    'create_interaction_response', 'get_interaction_response', 
    'update_interaction_response', 
    'delete_interaction_response', 'create_followup_message', 
    'get_followup_message', 'update_followup_message', 
    'delete_followup_message', 
    'get_application_role_connection_metadata', 
    'update_application_role_connection_metadata', 
    'get_guild_audit_log', 'get_guild_auto_moderation_rules', 
    'get_auto_moderation_rule', 'create_auto_moderation_rule', 
    'update_auto_moderation_rule', 'delete_auto_moderation_rule', 
    'get_channel', 'update_channel', 'delete_channel', 'get_messages', 
    'get_message', 'create_message', 'create_message_crosspost', 
    'create_reaction', 'delete_own_reaction', 'delete_user_reaction', 
    'get_reactions', 'delete_all_reactions', 'delete_all_emoji_reactions', 
    'update_message', 'delete_message', 'delete_messages', 
    'update_channel_permissions', 'get_channel_invites', 
    'create_channel_invite', 'delete_channel_permission', 
    'create_channel_follow', 'create_typing_indicator', 'get_channel_pins', 
    'create_channel_pin', 'delete_channel_pin', 'create_channel_recipient', 
    'delete_channel_recipient', 'create_message_thread', 'create_thread', 
    'create_self_thread_member', 'create_thread_member', 
    'delete_self_thread_member', 'delete_thread_member', 'get_thread_member', 
    'get_thread_members', 'get_public_archived_threads', 
    'get_private_archived_threads', 'get_self_private_archived_threads', 
    'get_guild_emojis', 'get_guild_emoji', 'create_guild_emoji', 
    'update_guild_emoji', 'delete_guild_emoji', 'create_guild', 'get_guild', 
    'get_guild_preview', 'update_guild', 'delete_guild', 'get_guild_channels', 
    'create_guild_channel', 'update_guild_channel_positions', 
    'get_active_guild_threads', 'get_guild_member', 'get_guild_members', 
    'search_guild_members', 'create_guild_member', 'update_guild_member', 
    'update_self_guild_member', 'create_guild_member_role', 
    'delete_guild_member_role', 'delete_guild_member', 'get_guild_bans', 
    'get_guild_ban', 'create_guild_ban', 'delete_guild_ban', 'get_guild_roles', 
    'create_guild_role', 'update_guild_role_positions', 'update_guild_role', 
    'update_guild_mfa_level', 'delete_guild_role', 'get_guild_prune_count', 
    'start_guild_prune', 'get_guild_voice_regions', 'get_guild_invites', 
    'get_guild_integrations', 'delete_guild_integration', 
    'get_guild_widget_settings', 'update_guild_widget', 'get_guild_widget', 
    'get_guild_vanity_url', 'get_guild_widget_image', 
    'get_guild_welcome_screen', 'update_guild_welcome_screen', 
    'get_guild_onboarding', 'update_self_voice_state', 'update_voice_state', 
    'get_guild_scheduled_events', 'create_guild_scheduled_event', 
    'get_guild_scheduled_event', 'update_guild_scheduled_event', 
    'delete_guild_scheduled_event', 'get_guild_scheduled_event_users', 
    'get_guild_template', 'create_guild_via_guild_template', 
    'get_guild_templates', 'create_guild_template', 'sync_guild_template', 
    'update_guild_template', 'delete_guild_template', 'get_invite',
    'delete_invite', 'create_stage_instance', 'get_stage_instance', 
    'update_stage_instance', 'delete_stage_instance', 'get_sticker', 
    'get_sticker_packs', 'get_guild_stickers', 'get_guild_sticker', 
    'create_guild_sticker', 'update_guild_sticker', 'delete_guild_sticker', 
    'get_self_user', 'get_user', 'update_self_user', 'get_self_guilds', 
    'get_self_guild_member', 'delete_self_guild_member', 'create_self_channel', 
    # 'create_group_channel', 
    'get_self_connections', 'get_self_application_role_connection', 
    'update_self_application_role_connection', 'get_voice_regions', 
    'create_webhook', 'get_channel_webhooks', 'get_guild_webhooks', 
    'get_webhook', 'get_webhook_via_token', 'update_webhook', 
    'update_webhook_via_token', 'delete_webhook', 'delete_webhook_via_token', 
    'create_webhook_message', 'create_webhook_message_slack_compatible', 
    'create_webhook_message_github_compatible', 'get_webhook_message', 
    'update_webhook_message', 'delete_webhook_message', 'get_gateway', 
    'get_gateway_bot', 'get_self_application_information',  
    'get_self_authorization_information'
)


class Route:
    
    """
    Contains all necessary data to perform a targeted request.

    :param verb:
        The HTTP method.
    :param path_template:
        The formattable path portion for the url.

    .. code-block:: python

        route = Route('GET', '/path/{0}/to/{1}/resource')
        value = await route(client, 'step0', 'step1', json = ..., params = ..., headers = ...)
    """

    __slots__ = ('_verb', '_path_template')

    def __init__(self, 
                 verb         : str,
                 path_template: str):

        self._verb = verb
        self._path_template = path_template

    @property
    def _identity(self):

        return (self._verb, self._path_template)

    def __call__(self, 
                 client: _client.Client, 
                 /,
                 *path_sections, 
                 **kwargs):

        path = self._path_template.format(*path_sections)

        return client.request(self._verb, path, **kwargs, identity = self._identity)
    
    def __repr__(self):

        return f'<{self.__class__.__name__}({self._verb} {self._path_template})>'
    

get_global_application_commands = Route('GET', '/applications/{0}/commands')
"""
|dsrc|
:ddoc:`Get Global Application Commands </interactions/application-commands#get-global-application-commands>`
"""

create_global_application_command = Route('POST', '/applications/{0}/commands')
"""
|dsrc|
:ddoc:`Create Global Application Command </interactions/application-commands#create-global-application-command>`
"""

get_global_application_command = Route('GET', '/applications/{0}/commands/{1}')
"""
|dsrc|
:ddoc:`Get Global Application Command </interactions/application-commands#get-global-application-command>`
"""

update_global_application_command = Route('PATCH', '/applications/{0}/commands/{1}')
"""
|dsrc|
:ddoc:`Edit Global Application Command </interactions/application-commands#edit-global-application-command>`
"""

delete_global_application_command = Route('DELETE', '/applications/{0}/commands/{1}')
"""
|dsrc|
:ddoc:`Delete Global Application Command </interactions/application-commands#delete-global-application-command>`
"""

update_all_global_application_commands = Route('PUT', '/applications/{0}/commands')
"""
|dsrc|
:ddoc:`Bulk Overwrite Global Application Commands </interactions/application-commands#bulk-overwrite-global-application-commands>`
"""

get_guild_application_commands = Route('GET', '/applications/{0}/guilds/{1}/commands')
"""
|dsrc|
:ddoc:`Get Guild Application Commands </interactions/application-commands#get-guild-application-commands>`
"""

create_guild_application_command = Route('POST', '/applications/{0}/guilds/{1}/commands')
"""
|dsrc|
:ddoc:`Create Guild Application Command </interactions/application-commands#create-guild-application-command>`
"""

get_guild_application_command = Route('GET', '/applications/{0}/guilds/{1}/commands/{2}')
"""
|dsrc|
:ddoc:`Get Guild Application Command </interactions/application-commands#get-guild-application-command>`
"""

update_guild_application_command = Route('PATCH', '/applications/{0}/guilds/{1}/commands/{2}')
"""
|dsrc|
:ddoc:`Edit Guild Application Command </interactions/application-commands#edit-guild-application-command>`
"""

delete_guild_application_command = Route('DELETE', '/applications/{0}/guilds/{1}/commands/{2}')
"""
|dsrc|
:ddoc:`Delete Guild Application Command </interactions/application-commands#delete-guild-application-command>`
"""

update_all_guild_application_commands = Route('PUT', '/applications/{0}/guilds/{1}/commands')
"""
|dsrc|
:ddoc:`Bulk Overwrite Guild Application Commands </interactions/application-commands#bulk-overwrite-guild-application-commands>`
"""

get_guild_application_command_permissions = Route('GET', '/applications/{0}/guilds/{1}/commands/permissions')
"""
|dsrc|
:ddoc:`Get Guild Application Command Permissions </interactions/application-commands#get-guild-application-command-permissions>`
"""

get_application_command_permissions = Route('GET', '/applications/{0}/guilds/{1}/commands/{2}/permissions')
"""
|dsrc|
:ddoc:`Get Application Command Permissions </interactions/application-commands#get-application-command-permissions>`
"""

update_application_command_permissions = Route('PUT', '/applications/{0}/guilds/{1}/commands/{2}/permissions')
"""
|dsrc|
:ddoc:`Edit Application Command Permissions </interactions/application-commands#edit-application-command-permissions>`
"""

# update_all_application_command_permissions = Route('PUT', '/applications/{0}/guilds/{1}/commands/permissions')
# """
# |dsrc|
# :ddoc:`Batch Edit Application Command Permissions </interactions/application-commands#batch-edit-application-command-permissions>`
# """

create_interaction_response = Route('POST', '/interactions/{0}/{1}/callback')
"""
|dsrc|
:ddoc:`Create Interaction Response </interactions/receiving-and-responding#create-interaction-response>`
"""

get_interaction_response = Route('GET', '/webhooks/{0}/{1}/messages/@original')
"""
|dsrc|
:ddoc:`Get Original Interaction Response </interactions/receiving-and-responding#get-original-interaction-response>`
"""

update_interaction_response = Route('PATCH', '/webhooks/{0}/{1}/messages/@original')
"""
|dsrc|
:ddoc:`Edit Original Interaction Response </interactions/receiving-and-responding#edit-original-interaction-response>`
"""

delete_interaction_response = Route('DELETE', '/webhooks/{0}/{1}/messages/@original')
"""
|dsrc|
:ddoc:`Delete Original Interaction Response </interactions/receiving-and-responding#delete-original-interaction-response>`
"""

create_followup_message = Route('POST', '/webhooks/{0}/{1}')
"""
|dsrc|
:ddoc:`Create Followup Message </interactions/receiving-and-responding#create-followup-message>`
"""

get_followup_message = Route('GET', '/webhooks/{0}/{1}/messages/{2}')
"""
|dsrc|
:ddoc:`Get Followup Message </interactions/receiving-and-responding#get-followup-message>`
"""

update_followup_message = Route('PATCH', '/webhooks/{0}/{1}/messages/{2}')
"""
|dsrc|
:ddoc:`Edit Followup Message </interactions/receiving-and-responding#edit-followup-message>`
"""

delete_followup_message = Route('DELETE', '/webhooks/{0}/{1}/messages/{2}')
"""
|dsrc|
:ddoc:`Delete Followup Message </interactions/receiving-and-responding#delete-followup-message>`
"""

get_application_role_connection_metadata = Route('GET', '/applications/{0}/role-connections/metadata')
"""
|dsrc|
:ddoc:`Get Application Role Connection Metadata Records </resources/application-role-connection-metadata#get-application-role-connection-metadata-records>`
"""

update_application_role_connection_metadata = Route('PUT', '/applications/{0}/role-connections/metadata')
"""
|dsrc|
:ddoc:`Update Application Role Connection Metadata Records </resources/application-role-connection-metadata#update-application-role-connection-metadata-records>`
"""

get_guild_audit_log = Route('GET', '/guilds/{0}/audit-logs')
"""
|dsrc|
:ddoc:`Get Guild Audit Log </resources/audit-log#get-guild-audit-log>`
"""

get_guild_auto_moderation_rules = Route('GET', '/guilds/{0}/auto-moderation/rules')
"""
|dsrc|
:ddoc:`List Auto Moderation Rules For Guild </resources/auto-moderation#list-auto-moderation-rules-for-guild>`
"""

get_auto_moderation_rule = Route('GET', '/guilds/{0}/auto-moderation/rules/{1}')
"""
|dsrc|
:ddoc:`Get Auto Moderation Rule </resources/auto-moderation#get-auto-moderation-rule>`
"""

create_auto_moderation_rule = Route('POST', '/guilds/{0}/auto-moderation/rules')
"""
|dsrc|
:ddoc:`Create Auto Moderation Rule </resources/auto-moderation#create-auto-moderation-rule>`
"""

update_auto_moderation_rule = Route('PATCH', '/guilds/{0}/auto-moderation/rules/{1}')
"""
|dsrc|
:ddoc:`Modify Auto Moderation Rule </resources/auto-moderation#modify-auto-moderation-rule>`
"""

delete_auto_moderation_rule = Route('DELETE', '/guilds/{0}/auto-moderation/rules/{1}')
"""
|dsrc|
:ddoc:`Delete Auto Moderation Rule </resources/auto-moderation#delete-auto-moderation-rule>`
"""

get_channel = Route('GET', '/channels/{0}')
"""
|dsrc|
:ddoc:`Get Channel </resources/channel#get-channel>`
"""

update_channel = Route('PATCH', '/channels/{0}')
"""
|dsrc|
:ddoc:`Modify Channel </resources/channel#modify-channel>`
"""

delete_channel = Route('DELETE', '/channels/{0}')
"""
|dsrc|
:ddoc:`Delete/Close Channel </resources/channel#delete/close-channel>`
"""

get_messages = Route('GET', '/channels/{0}/messages')
"""
|dsrc|
:ddoc:`Get Channel Messages </resources/channel#get-channel-messages>`
"""

get_message = Route('GET', '/channels/{0}/messages/{1}')
"""
|dsrc|
:ddoc:`Get Channel Message </resources/channel#get-channel-message>`
"""

create_message = Route('POST', '/channels/{0}/messages')
"""
|dsrc|
:ddoc:`Create Message </resources/channel#create-message>`
"""

create_message_crosspost = Route('POST', '/channels/{0}/messages/{1}/crosspost')
"""
|dsrc|
:ddoc:`Crosspost Message </resources/channel#crosspost-message>`
"""

create_reaction = Route('PUT', '/channels/{0}/messages/{1}/reactions/{2}/@me')
"""
|dsrc|
:ddoc:`Create Reaction </resources/channel#create-reaction>`
"""

delete_own_reaction = Route('DELETE', '/channels/{0}/messages/{1}/reactions/{2}/@me')
"""
|dsrc|
:ddoc:`Delete Own Reaction </resources/channel#delete-own-reaction>`
"""

delete_user_reaction = Route('DELETE', '/channels/{0}/messages/{1}/reactions/{2}/{3}')
"""
|dsrc|
:ddoc:`Delete User Reaction </resources/channel#delete-user-reaction>`
"""

get_reactions = Route('GET', '/channels/{0}/messages/{1}/reactions/{2}')
"""
|dsrc|
:ddoc:`Get Reactions </resources/channel#get-reactions>`
"""

delete_all_reactions = Route('DELETE', '/channels/{0}/messages/{1}/reactions')
"""
|dsrc|
:ddoc:`Delete All Reactions </resources/channel#delete-all-reactions>`
"""

delete_all_emoji_reactions = Route('DELETE', '/channels/{0}/messages/{1}/reactions/{2}')
"""
|dsrc|
:ddoc:`Delete All Reactions For Emoji </resources/channel#delete-all-reactions-for-emoji>`
"""

update_message = Route('PATCH', '/channels/{0}/messages/{1}')
"""
|dsrc|
:ddoc:`Edit Message </resources/channel#edit-message>`
"""

delete_message = Route('DELETE', '/channels/{0}/messages/{1}')
"""
|dsrc|
:ddoc:`Delete Message </resources/channel#delete-message>`
"""

delete_messages = Route('POST', '/channels/{0}/messages/bulk-delete')
"""
|dsrc|
:ddoc:`Bulk Delete Messages </resources/channel#bulk-delete-messages>`
"""

update_channel_permissions = Route('PUT', '/channels/{0}/permissions/{1}')
"""
|dsrc|
:ddoc:`Edit Channel Permissions </resources/channel#edit-channel-permissions>`
"""

get_channel_invites = Route('GET', '/channels/{0}/invites')
"""
|dsrc|
:ddoc:`Get Channel Invites </resources/channel#get-channel-invites>`
"""

create_channel_invite = Route('POST', '/channels/{0}/invites')
"""
|dsrc|
:ddoc:`Create Channel Invite </resources/channel#create-channel-invite>`
"""

delete_channel_permission = Route('DELETE', '/channels/{0}/permissions/{1}')
"""
|dsrc|
:ddoc:`Delete Channel Permission </resources/channel#delete-channel-permission>`
"""

create_channel_follow = Route('POST', '/channels/{0}/followers')
"""
|dsrc|
:ddoc:`Follow Announcement Channel </resources/channel#follow-announcement-channel>`
"""

create_typing_indicator = Route('POST', '/channels/{0}/typing')
"""
|dsrc|
:ddoc:`Trigger Typing Indicator </resources/channel#trigger-typing-indicator>`
"""

get_channel_pins = Route('GET', '/channels/{0}/pins')
"""
|dsrc|
:ddoc:`Get Pinned Messages </resources/channel#get-pinned-messages>`
"""

create_channel_pin = Route('PUT', '/channels/{0}/pins/{1}')
"""
|dsrc|
:ddoc:`Pin Message </resources/channel#pin-message>`
"""

delete_channel_pin = Route('DELETE', '/channels/{0}/pins/{1}')
"""
|dsrc|
:ddoc:`Unpin Message </resources/channel#unpin-message>`
"""

create_channel_recipient = Route('PUT', '/channels/{0}/recipients/{1}')
"""
|dsrc|
:ddoc:`Group Dm Add Recipient </resources/channel#group-dm-add-recipient>`
"""

delete_channel_recipient = Route('DELETE', '/channels/{0}/recipients/{1}')
"""
|dsrc|
:ddoc:`Group Dm Remove Recipient </resources/channel#group-dm-remove-recipient>`
"""

create_message_thread = Route('POST', '/channels/{0}/messages/{1}/threads')
"""
|dsrc|
:ddoc:`Start Thread From Message </resources/channel#start-thread-from-message>`
"""

create_thread = Route('POST', '/channels/{0}/threads')
"""
|dsrc|
:ddoc:`Start Thread Without Message </resources/channel#start-thread-without-message>`
"""

create_self_thread_member = Route('PUT', '/channels/{0}/thread-members/@me')
"""
|dsrc|
:ddoc:`Join Thread </resources/channel#join-thread>`
"""

create_thread_member = Route('PUT', '/channels/{0}/thread-members/{1}')
"""
|dsrc|
:ddoc:`Add Thread Member </resources/channel#add-thread-member>`
"""

delete_self_thread_member = Route('DELETE', '/channels/{0}/thread-members/@me')
"""
|dsrc|
:ddoc:`Leave Thread </resources/channel#leave-thread>`
"""

delete_thread_member = Route('DELETE', '/channels/{0}/thread-members/{1}')
"""
|dsrc|
:ddoc:`Remove Thread Member </resources/channel#remove-thread-member>`
"""

get_thread_member = Route('GET', '/channels/{0}/thread-members/{1}')
"""
|dsrc|
:ddoc:`Get Thread Member </resources/channel#get-thread-member>`
"""

get_thread_members = Route('GET', '/channels/{0}/thread-members')
"""
|dsrc|
:ddoc:`List Thread Members </resources/channel#list-thread-members>`
"""

get_public_archived_threads = Route('GET', '/channels/{0}/threads/archived/public')
"""
|dsrc|
:ddoc:`List Public Archived Threads </resources/channel#list-public-archived-threads>`
"""

get_private_archived_threads = Route('GET', '/channels/{0}/threads/archived/private')
"""
|dsrc|
:ddoc:`List Private Archived Threads </resources/channel#list-private-archived-threads>`
"""

get_self_private_archived_threads = Route('GET', '/channels/{0}/users/@me/threads/archived/private')
"""
|dsrc|
:ddoc:`List Joined Private Archived Threads </resources/channel#list-joined-private-archived-threads>`
"""

get_guild_emojis = Route('GET', '/guilds/{0}/emojis')
"""
|dsrc|
:ddoc:`List Guild Emojis </resources/emoji#list-guild-emojis>`
"""

get_guild_emoji = Route('GET', '/guilds/{0}/emojis/{1}')
"""
|dsrc|
:ddoc:`Get Guild Emoji </resources/emoji#get-guild-emoji>`
"""

create_guild_emoji = Route('POST', '/guilds/{0}/emojis')
"""
|dsrc|
:ddoc:`Create Guild Emoji </resources/emoji#create-guild-emoji>`
"""

update_guild_emoji = Route('PATCH', '/guilds/{0}/emojis/{1}')
"""
|dsrc|
:ddoc:`Modify Guild Emoji </resources/emoji#modify-guild-emoji>`
"""

delete_guild_emoji = Route('DELETE', '/guilds/{0}/emojis/{1}')
"""
|dsrc|
:ddoc:`Delete Guild Emoji </resources/emoji#delete-guild-emoji>`
"""

create_guild = Route('POST', '/guilds')
"""
|dsrc|
:ddoc:`Create Guild </resources/guild#create-guild>`
"""

get_guild = Route('GET', '/guilds/{0}')
"""
|dsrc|
:ddoc:`Get Guild </resources/guild#get-guild>`
"""

get_guild_preview = Route('GET', '/guilds/{0}/preview')
"""
|dsrc|
:ddoc:`Get Guild Preview </resources/guild#get-guild-preview>`
"""

update_guild = Route('PATCH', '/guilds/{0}')
"""
|dsrc|
:ddoc:`Modify Guild </resources/guild#modify-guild>`
"""

delete_guild = Route('DELETE', '/guilds/{0}')
"""
|dsrc|
:ddoc:`Delete Guild </resources/guild#delete-guild>`
"""

get_guild_channels = Route('GET', '/guilds/{0}/channels')
"""
|dsrc|
:ddoc:`Get Guild Channels </resources/guild#get-guild-channels>`
"""

create_guild_channel = Route('POST', '/guilds/{0}/channels')
"""
|dsrc|
:ddoc:`Create Guild Channel </resources/guild#create-guild-channel>`
"""

update_guild_channel_positions = Route('PATCH', '/guilds/{0}/channels')
"""
|dsrc|
:ddoc:`Modify Guild Channel Positions </resources/guild#modify-guild-channel-positions>`
"""

get_active_guild_threads = Route('GET', '/guilds/{0}/threads/active')
"""
|dsrc|
:ddoc:`List Active Guild Threads </resources/guild#list-active-guild-threads>`
"""

get_guild_member = Route('GET', '/guilds/{0}/members/{1}')
"""
|dsrc|
:ddoc:`Get Guild Member </resources/guild#get-guild-member>`
"""

get_guild_members = Route('GET', '/guilds/{0}/members')
"""
|dsrc|
:ddoc:`List Guild Members </resources/guild#list-guild-members>`
"""

search_guild_members = Route('GET', '/guilds/{0}/members/search')
"""
|dsrc|
:ddoc:`Search Guild Members </resources/guild#search-guild-members>`
"""

create_guild_member = Route('PUT', '/guilds/{0}/members/{1}')
"""
|dsrc|
:ddoc:`Add Guild Member </resources/guild#add-guild-member>`
"""

update_guild_member = Route('PATCH', '/guilds/{0}/members/{1}')
"""
|dsrc|
:ddoc:`Modify Guild Member </resources/guild#modify-guild-member>`
"""

update_self_guild_member = Route('PATCH', '/guilds/{0}/members/@me')
"""
|dsrc|
:ddoc:`Modify Current Member </resources/guild#modify-current-member>`
"""

create_guild_member_role = Route('PUT', '/guilds/{0}/members/{1}/roles/{2}')
"""
|dsrc|
:ddoc:`Add Guild Member Role </resources/guild#add-guild-member-role>`
"""

delete_guild_member_role = Route('DELETE', '/guilds/{0}/members/{1}/roles/{2}')
"""
|dsrc|
:ddoc:`Remove Guild Member Role </resources/guild#remove-guild-member-role>`
"""

delete_guild_member = Route('DELETE', '/guilds/{0}/members/{1}')
"""
|dsrc|
:ddoc:`Remove Guild Member </resources/guild#remove-guild-member>`
"""

get_guild_bans = Route('GET', '/guilds/{0}/bans')
"""
|dsrc|
:ddoc:`Get Guild Bans </resources/guild#get-guild-bans>`
"""

get_guild_ban = Route('GET', '/guilds/{0}/bans/{1}')
"""
|dsrc|
:ddoc:`Get Guild Ban </resources/guild#get-guild-ban>`
"""

create_guild_ban = Route('PUT', '/guilds/{0}/bans/{1}')
"""
|dsrc|
:ddoc:`Create Guild Ban </resources/guild#create-guild-ban>`
"""

delete_guild_ban = Route('DELETE', '/guilds/{0}/bans/{1}')
"""
|dsrc|
:ddoc:`Remove Guild Ban </resources/guild#remove-guild-ban>`
"""

get_guild_roles = Route('GET', '/guilds/{0}/roles')
"""
|dsrc|
:ddoc:`Get Guild Roles </resources/guild#get-guild-roles>`
"""

create_guild_role = Route('POST', '/guilds/{0}/roles')
"""
|dsrc|
:ddoc:`Create Guild Role </resources/guild#create-guild-role>`
"""

update_guild_role_positions = Route('PATCH', '/guilds/{0}/roles')
"""
|dsrc|
:ddoc:`Modify Guild Role Positions </resources/guild#modify-guild-role-positions>`
"""

update_guild_role = Route('PATCH', '/guilds/{0}/roles/{1}')
"""
|dsrc|
:ddoc:`Modify Guild Role </resources/guild#modify-guild-role>`
"""

update_guild_mfa_level = Route('POST', '/guilds/{0}/mfa')
"""
|dsrc|
:ddoc:`Modify Guild Mfa Level </resources/guild#modify-guild-mfa-level>`
"""

delete_guild_role = Route('DELETE', '/guilds/{0}/roles/{1}')
"""
|dsrc|
:ddoc:`Delete Guild Role </resources/guild#delete-guild-role>`
"""

get_guild_prune_count = Route('GET', '/guilds/{0}/prune')
"""
|dsrc|
:ddoc:`Get Guild Prune Count </resources/guild#get-guild-prune-count>`
"""

start_guild_prune = Route('POST', '/guilds/{0}/prune')
"""
|dsrc|
:ddoc:`Begin Guild Prune </resources/guild#begin-guild-prune>`
"""

get_guild_voice_regions = Route('GET', '/guilds/{0}/regions')
"""
|dsrc|
:ddoc:`Get Guild Voice Regions </resources/guild#get-guild-voice-regions>`
"""

get_guild_invites = Route('GET', '/guilds/{0}/invites')
"""
|dsrc|
:ddoc:`Get Guild Invites </resources/guild#get-guild-invites>`
"""

get_guild_integrations = Route('GET', '/guilds/{0}/integrations')
"""
|dsrc|
:ddoc:`Get Guild Integrations </resources/guild#get-guild-integrations>`
"""

delete_guild_integration = Route('DELETE', '/guilds/{0}/integrations/{1}')
"""
|dsrc|
:ddoc:`Delete Guild Integration </resources/guild#delete-guild-integration>`
"""

get_guild_widget_settings = Route('GET', '/guilds/{0}/widget')
"""
|dsrc|
:ddoc:`Get Guild Widget Settings </resources/guild#get-guild-widget-settings>`
"""

update_guild_widget = Route('PATCH', '/guilds/{0}/widget')
"""
|dsrc|
:ddoc:`Modify Guild Widget </resources/guild#modify-guild-widget>`
"""

get_guild_widget = Route('GET', '/guilds/{0}/widget.json')
"""
|dsrc|
:ddoc:`Get Guild Widget </resources/guild#get-guild-widget>`
"""

get_guild_vanity_url = Route('GET', '/guilds/{0}/vanity-url')
"""
|dsrc|
:ddoc:`Get Guild Vanity Url </resources/guild#get-guild-vanity-url>`
"""

get_guild_widget_image = Route('GET', '/guilds/{0}/widget.png')
"""
|dsrc|
:ddoc:`Get Guild Widget Image </resources/guild#get-guild-widget-image>`
"""

get_guild_welcome_screen = Route('GET', '/guilds/{0}/welcome-screen')
"""
|dsrc|
:ddoc:`Get Guild Welcome Screen </resources/guild#get-guild-welcome-screen>`
"""

update_guild_welcome_screen = Route('PATCH', '/guilds/{0}/welcome-screen')
"""
|dsrc|
:ddoc:`Modify Guild Welcome Screen </resources/guild#modify-guild-welcome-screen>`
"""

get_guild_onboarding = Route('GET', '/guilds/{0}/onboarding')
"""
|dsrc|
:ddoc:`Get Guild Onboarding </resources/guild#get-guild-onboarding>`
"""

update_self_voice_state = Route('PATCH', '/guilds/{0}/voice-states/@me')
"""
|dsrc|
:ddoc:`Modify Current User Voice State </resources/guild#modify-current-user-voice-state>`
"""

update_voice_state = Route('PATCH', '/guilds/{0}/voice-states/{1}')
"""
|dsrc|
:ddoc:`Modify User Voice State </resources/guild#modify-user-voice-state>`
"""

get_guild_scheduled_events = Route('GET', '/guilds/{0}/scheduled-events')
"""
|dsrc|
:ddoc:`List Scheduled Events For Guild </resources/guild-scheduled-event#list-scheduled-events-for-guild>`
"""

create_guild_scheduled_event = Route('POST', '/guilds/{0}/scheduled-events')
"""
|dsrc|
:ddoc:`Create Guild Scheduled Event </resources/guild-scheduled-event#create-guild-scheduled-event>`
"""

get_guild_scheduled_event = Route('GET', '/guilds/{0}/scheduled-events/{1}')
"""
|dsrc|
:ddoc:`Get Guild Scheduled Event </resources/guild-scheduled-event#get-guild-scheduled-event>`
"""

update_guild_scheduled_event = Route('PATCH', '/guilds/{0}/scheduled-events/{1}')
"""
|dsrc|
:ddoc:`Modify Guild Scheduled Event </resources/guild-scheduled-event#modify-guild-scheduled-event>`
"""

delete_guild_scheduled_event = Route('DELETE', '/guilds/{0}/scheduled-events/{1}')
"""
|dsrc|
:ddoc:`Delete Guild Scheduled Event </resources/guild-scheduled-event#delete-guild-scheduled-event>`
"""

get_guild_scheduled_event_users = Route('GET', '/guilds/{0}/scheduled-events/{1}/users')
"""
|dsrc|
:ddoc:`Get Guild Scheduled Event Users </resources/guild-scheduled-event#get-guild-scheduled-event-users>`
"""

get_guild_template = Route('GET', '/guilds/templates/{0}')
"""
|dsrc|
:ddoc:`Get Guild Template </resources/guild-template#get-guild-template>`
"""

create_guild_via_guild_template = Route('POST', '/guilds/templates/{0}')
"""
|dsrc|
:ddoc:`Create Guild From Guild Template </resources/guild-template#create-guild-from-guild-template>`
"""

get_guild_templates = Route('GET', '/guilds/{0}/templates')
"""
|dsrc|
:ddoc:`Get Guild Templates </resources/guild-template#get-guild-templates>`
"""

create_guild_template = Route('POST', '/guilds/{0}/templates')
"""
|dsrc|
:ddoc:`Create Guild Template </resources/guild-template#create-guild-template>`
"""

sync_guild_template = Route('PUT', '/guilds/{0}/templates/{1}')
"""
|dsrc|
:ddoc:`Sync Guild Template </resources/guild-template#sync-guild-template>`
"""

update_guild_template = Route('PATCH', '/guilds/{0}/templates/{1}')
"""
|dsrc|
:ddoc:`Modify Guild Template </resources/guild-template#modify-guild-template>`
"""

delete_guild_template = Route('DELETE', '/guilds/{0}/templates/{1}')
"""
|dsrc|
:ddoc:`Delete Guild Template </resources/guild-template#delete-guild-template>`
"""

get_invite = Route('GET', '/invites/{0}')
"""
|dsrc|
:ddoc:`Get Invite </resources/invite#get-invite>`
"""

delete_invite = Route('DELETE', '/invites/{0}')
"""
|dsrc|
:ddoc:`Delete Invite </resources/invite#delete-invite>`
"""

create_stage_instance = Route('POST', '/stage-instances')
"""
|dsrc|
:ddoc:`Create Stage Instance </resources/stage-instance#create-stage-instance>`
"""

get_stage_instance = Route('GET', '/stage-instances/{0}')
"""
|dsrc|
:ddoc:`Get Stage Instance </resources/stage-instance#get-stage-instance>`
"""

update_stage_instance = Route('PATCH', '/stage-instances/{0}')
"""
|dsrc|
:ddoc:`Modify Stage Instance </resources/stage-instance#modify-stage-instance>`
"""

delete_stage_instance = Route('DELETE', '/stage-instances/{0}')
"""
|dsrc|
:ddoc:`Delete Stage Instance </resources/stage-instance#delete-stage-instance>`
"""

get_sticker = Route('GET', '/stickers/{0}')
"""
|dsrc|
:ddoc:`Get Sticker </resources/sticker#get-sticker>`
"""

get_sticker_packs = Route('GET', '/sticker-packs')
"""
|dsrc|
:ddoc:`List Nitro Sticker Packs </resources/sticker#list-nitro-sticker-packs>`
"""

get_guild_stickers = Route('GET', '/guilds/{0}/stickers')
"""
|dsrc|
:ddoc:`List Guild Stickers </resources/sticker#list-guild-stickers>`
"""

get_guild_sticker = Route('GET', '/guilds/{0}/stickers/{1}')
"""
|dsrc|
:ddoc:`Get Guild Sticker </resources/sticker#get-guild-sticker>`
"""

create_guild_sticker = Route('POST', '/guilds/{0}/stickers')
"""
|dsrc|
:ddoc:`Create Guild Sticker </resources/sticker#create-guild-sticker>`
"""

update_guild_sticker = Route('PATCH', '/guilds/{0}/stickers/{1}')
"""
|dsrc|
:ddoc:`Modify Guild Sticker </resources/sticker#modify-guild-sticker>`
"""

delete_guild_sticker = Route('DELETE', '/guilds/{0}/stickers/{1}')
"""
|dsrc|
:ddoc:`Delete Guild Sticker </resources/sticker#delete-guild-sticker>`
"""

get_self_user = Route('GET', '/users/@me')
"""
|dsrc|
:ddoc:`Get Current User </resources/user#get-current-user>`
"""

get_user = Route('GET', '/users/{0}')
"""
|dsrc|
:ddoc:`Get User </resources/user#get-user>`
"""

update_self_user = Route('PATCH', '/users/@me')
"""
|dsrc|
:ddoc:`Modify Current User </resources/user#modify-current-user>`
"""

get_self_guilds = Route('GET', '/users/@me/guilds')
"""
|dsrc|
:ddoc:`Get Current User Guilds </resources/user#get-current-user-guilds>`
"""

get_self_guild_member = Route('GET', '/users/@me/guilds/{0}/member')
"""
|dsrc|
:ddoc:`Get Current User Guild Member </resources/user#get-current-user-guild-member>`
"""

delete_self_guild_member = Route('DELETE', '/users/@me/guilds/{0}')
"""
|dsrc|
:ddoc:`Leave Guild </resources/user#leave-guild>`
"""

create_self_channel = Route('POST', '/users/@me/channels')
"""
|dsrc|
:ddoc:`Create Dm </resources/user#create-dm>`
"""

# create_group_channel = Route('POST', '/users/@me/channels')
# """
# |dsrc|
# :ddoc:`Create Group Dm </resources/user#create-group-dm>`
# """

get_self_connections = Route('GET', '/users/@me/connections')
"""
|dsrc|
:ddoc:`Get User Connections </resources/user#get-user-connections>`
"""

get_self_application_role_connection = Route('GET', '/users/@me/applications/{0}/role-connection')
"""
|dsrc|
:ddoc:`Get User Application Role Connection </resources/user#get-user-application-role-connection>`
"""

update_self_application_role_connection = Route('PUT', '/users/@me/applications/{0}/role-connection')
"""
|dsrc|
:ddoc:`Update User Application Role Connection </resources/user#update-user-application-role-connection>`
"""

get_voice_regions = Route('GET', '/voice/regions')
"""
|dsrc|
:ddoc:`List Voice Regions </resources/voice#list-voice-regions>`
"""

create_webhook = Route('POST', '/channels/{0}/webhooks')
"""
|dsrc|
:ddoc:`Create Webhook </resources/webhook#create-webhook>`
"""

get_channel_webhooks = Route('GET', '/channels/{0}/webhooks')
"""
|dsrc|
:ddoc:`Get Channel Webhooks </resources/webhook#get-channel-webhooks>`
"""

get_guild_webhooks = Route('GET', '/guilds/{0}/webhooks')
"""
|dsrc|
:ddoc:`Get Guild Webhooks </resources/webhook#get-guild-webhooks>`
"""

get_webhook = Route('GET', '/webhooks/{0}')
"""
|dsrc|
:ddoc:`Get Webhook </resources/webhook#get-webhook>`
"""

get_webhook_via_token = Route('GET', '/webhooks/{0}/{1}')
"""
|dsrc|
:ddoc:`Get Webhook With Token </resources/webhook#get-webhook-with-token>`
"""

update_webhook = Route('PATCH', '/webhooks/{0}')
"""
|dsrc|
:ddoc:`Modify Webhook </resources/webhook#modify-webhook>`
"""

update_webhook_via_token = Route('PATCH', '/webhooks/{0}/{1}')
"""
|dsrc|
:ddoc:`Modify Webhook With Token </resources/webhook#modify-webhook-with-token>`
"""

delete_webhook = Route('DELETE', '/webhooks/{0}')
"""
|dsrc|
:ddoc:`Delete Webhook </resources/webhook#delete-webhook>`
"""

delete_webhook_via_token = Route('DELETE', '/webhooks/{0}/{1}')
"""
|dsrc|
:ddoc:`Delete Webhook With Token </resources/webhook#delete-webhook-with-token>`
"""

create_webhook_message = Route('POST', '/webhooks/{0}/{1}')
"""
|dsrc|
:ddoc:`Execute Webhook </resources/webhook#execute-webhook>`
"""

create_webhook_message_slack_compatible = Route('POST', '/webhooks/{0}/{1}/slack')
"""
|dsrc|
:ddoc:`Execute Slack Compatible Webhook </resources/webhook#execute-slack-compatible-webhook>`
"""

create_webhook_message_github_compatible = Route('POST', '/webhooks/{0}/{1}/github')
"""
|dsrc|
:ddoc:`Execute Github Compatible Webhook </resources/webhook#execute-github-compatible-webhook>`
"""

get_webhook_message = Route('GET', '/webhooks/{0}/{1}/messages/{2}')
"""
|dsrc|
:ddoc:`Get Webhook Message </resources/webhook#get-webhook-message>`
"""

update_webhook_message = Route('PATCH', '/webhooks/{0}/{1}/messages/{2}')
"""
|dsrc|
:ddoc:`Edit Webhook Message </resources/webhook#edit-webhook-message>`
"""

delete_webhook_message = Route('DELETE', '/webhooks/{0}/{1}/messages/{2}')
"""
|dsrc|
:ddoc:`Delete Webhook Message </resources/webhook#delete-webhook-message>`
"""

get_gateway = Route('GET', '/gateway')
"""
|dsrc|
:ddoc:`Get Gateway </topics/gateway#get-gateway>`
"""

get_gateway_bot = Route('GET', '/gateway/bot')
"""
|dsrc|
:ddoc:`Get Gateway Bot </topics/gateway#get-gateway-bot>`
"""

get_self_application_information = Route('GET', '/oauth2/applications/@me')
"""
|dsrc|
:ddoc:`Get Current Bot Application Information </topics/oauth2#get-current-bot-application-information>`
"""

get_self_authorization_information = Route('GET', '/oauth2/@me')
"""
|dsrc|
:ddoc:`Get Current Authorization Information </topics/oauth2#get-current-authorization-information>`
"""


import enum


__all__ = ('OpCode',)


class OpCode(enum.IntEnum):

    """
    All possible send/receive opcodes.
    """

    dispatch               = 0
    heartbeat              = 1
    identify               = 2
    update_presence        = 3
    update_voice_state     = 4
    resume                 = 6
    reconnect              = 7
    request_guild_members  = 8
    invalid_session        = 9
    hello                  = 10
    heartbeat_ack          = 11




import asyncio
import aiocord

async def main(client):
    message = await client.create_message('864902189816273146', content = 'hello!')
    await client.create_reaction(message.channel_id, message.id, '🤠')

loop = asyncio.get_event_loop()

client = aiocord.client.Client(token = ...)

loop.run_until_complete(main(client))


import aiocord

async def __load__(info):
    print('loaded', __name__)

async def __drop__(info):
    print('dropped', __name__)

@aiocord.widget.callback(aiocord.events.CreateMessage)
async def _callback_create_message(info, core_event):
    print('message created!', core_event.message.content)

@aiocord.widget.interact('blep')
async def _interact_blep(info, core_event):
    users = core_event.interaction.data.resolved.users
    blepping = ' '.join(user.mention() for user in users)
    return aiocord.model.protocols.InteractionResponse(
        type = aiocord.model.enums.InteractionResponseType.channel_message_with_source,
        data = aiocord.model.protocols.MessageInteractionResponse(
            content = f'blep {blepping}'
        )
    )


import enum


__all__ = (
    'ApplicationCommandType', 'StatusType', 'Locale',
    'Permissions', 'ApplicationCommandOptionType', 'ChannelType', 
    'MessageComponentType', 'MessageButtonComponentStyle', 
    'MessageTextInputComponentStyle', 'InteractionType', 
    'InteractionResponseType', 'ApplicationFlags', 
    'ApplicationRoleConnectionMetadataType', 'AuditLogEvent', 
    'OptionalAuditLogEntryInfoOverwrittenEntityType', 
    'AutoModerationTriggerType', 'AutoModerationRuleKeywordPresetType', 
    'AutoModerationRuleEventType', 'AutoModerationActionType', 
    'ChannelVideoQualityMode', 'ChannelFlags', 'ChannelSortOrderType', 
    'ForumLayoutType', 'MessageType', 'MessageActivityType', 'MessageFlags', 
    'OverwriteType', 'EmbedType', 'AllowedMentionsType', 
    'GuildVerificationLevel', 'GuildDefaultMessageNotificationLevel', 
    'GuildExplicitContentFilterLevel', 'GuildFeature', 'GuildMFALevel', 
    'GuildNSFWLevel', 'GuildSystemChannelFlags', 'GuildPremiumTier', 
    'GuildMemberFlags', 'IntegrationExpireBehaviorType', 
    'GuildOnboardingPromptType', 'GuildScheduledEventPrivacyLevel', 
    'GuildScheduledEventEntityType', 'GuildScheduledEventStatus', 
    'InviteTargetType', 'StageInstancePrivacyLevel', 'StickerType', 
    'StickerFormatType', 'UserFlags', 'UserPremiumType', 
    'ConnectionVisibilityType', 'WebhookType', 'ActivityType', 
    'ActivityFlags', 'TeamMemberMembershipState', 'WidgetStyleOption',
    'SpeechFlags', 'TimestampStyle'
)


class EnumMeta(enum.EnumType):

    def __call__(self, data, *args, **kwargs):
        try:
            value = super().__call__(data, *args, **kwargs)
        except ValueError:
            value = data
        return value
    

class ApplicationCommandType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Application Command Option Types </interactions/application-commands#application-command-object-application-command-option-type>`
    """

    chat_input = 1
    user       = 2
    message    = 3


class StatusType(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Update Presence Status Types </topics/gateway-events#update-presence-status-types>`
    """

    online    = 'online'
    dnd       = 'dnd'
    idle      = 'idle'
    invisible = 'invisible'
    offline   = 'offline'


class Locale(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Locales </reference#locales>`
    """

    indonesian    = 'id'
    danish        = 'da'
    german        = 'de'
    english_uk    = 'en-GB'
    english_us    = 'en-US'
    spanish_es    = 'es-ES'
    french        = 'fr'
    croatian      = 'hr'
    italian       = 'it'
    lithuanian    = 'lt'
    hungarian     = 'hu'
    dutch         = 'nl'
    norwegian     = 'no'
    polish        = 'pl'
    portuguese_br = 'pt-BR'
    romanian      = 'ro'
    finnish       = 'fi'
    swedish_se    = 'sv-SE'
    vietnamese    = 'vi'
    turkish       = 'tr'
    czech         = 'cs'
    greek         = 'el'
    bulgarian     = 'bg'
    russian       = 'ru'
    ukrainian     = 'uk'
    hindi         = 'hi'
    thai          = 'th'
    chinese_cn    = 'zh-CN'
    japanese      = 'ja'
    chinese_tw    = 'zh-TW'
    korean        = 'ko'


class PermissionsMeta(enum.IntFlag, metaclass = EnumMeta.__class__):

    def __call__(self, data, *args, **kwargs):
        data = int(data)
        return super().__call__(data, *args, **kwargs)


class Permissions(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Permission Flags </topics/permissions#permissions-bitwise-permission-flags>`
    """
    
    create_instant_invite               = 1 << 0
    kick_members                        = 1 << 1
    ban_members                         = 1 << 2
    administrator                       = 1 << 3
    manage_channels                     = 1 << 4
    manage_guild                        = 1 << 5
    add_reactions                       = 1 << 6
    view_audit_log                      = 1 << 7
    priority_speaker                    = 1 << 8
    stream                              = 1 << 9
    view_channel                        = 1 << 10
    send_messages                       = 1 << 11
    send_tts_messages                   = 1 << 12
    manage_messages                     = 1 << 13
    embed_links                         = 1 << 14
    attach_files                        = 1 << 15
    read_message_history                = 1 << 16
    mention_everyone                    = 1 << 17
    use_external_emojis                 = 1 << 18
    view_guild_insights                 = 1 << 19
    connect                             = 1 << 20
    speak                               = 1 << 21
    mute_members                        = 1 << 22
    deafen_members                      = 1 << 23
    move_members                        = 1 << 24
    use_vad                             = 1 << 25
    change_nickname                     = 1 << 26
    manage_nicknames                    = 1 << 27
    manage_roles                        = 1 << 28
    manage_webhooks                     = 1 << 29
    manage_guild_expressions            = 1 << 30
    use_application_commands            = 1 << 31
    request_to_speak                    = 1 << 32
    manage_events                       = 1 << 33
    manage_threads                      = 1 << 34
    create_public_threads               = 1 << 35
    create_private_threads              = 1 << 36
    use_external_stickers               = 1 << 37
    send_messages_in_threads            = 1 << 38
    use_embedded_activities             = 1 << 39
    moderate_members                    = 1 << 40
    view_creator_monetization_analytics = 1 << 41
    use_soundboard                      = 1 << 42
    send_voice_messages                 = 1 << 46


class ApplicationCommandOptionType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Application Command Option Types </interactions/application-commands#application-command-object-application-command-option-type>`
    """

    sub_command       = 1
    sub_command_group = 2
    string            = 3
    integer           = 4
    boolean           = 5
    user              = 6
    channel           = 7
    role              = 8
    mentionable       = 9
    number            = 10
    attachment        = 11


class ChannelType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Channel Types </resources/channel#channel-object-channel-types>`
    """

    guild_text          = 0
    dm                  = 1
    guild_voice         = 2
    group_dm            = 3
    guild_category      = 4
    guild_announcement  = 5
    announcement_thread = 10
    public_thread       = 11
    private_thread      = 12
    guild_stage_voice   = 13
    guild_directory     = 14
    guild_forum         = 15



class ApplicationCommandPermissionType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Application Command Permission Types </interactions/application-commands#application-command-permissions-object-application-command-permission-type>`
    """

    role    = 1
    user    = 2
    channel = 3


class MessageComponentType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Component Types </interactions/message-components#component-object-component-types>`
    """

    action_row         = 1
    button             = 2
    string_select      = 3
    text_input         = 4
    user_select        = 5
    role_select        = 6
    mentionable_select = 7
    channel_select     = 8


class MessageButtonComponentStyle(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Button Styles </interactions/message-components#button-object-button-styles>`
    """

    primary   = 1
    secondary = 2
    success   = 3
    danger    = 4
    link      = 5


class MessageTextInputComponentStyle(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Text Inputs Text Input Styles </interactions/message-components#text-inputs-text-input-styles>`
    """

    short = 1
    paragraph = 2


class InteractionType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Interaction Types </interactions/receiving-and-responding#interaction-object-interaction-type>`
    """

    ping                             = 1
    application_command              = 2
    message_component                = 3
    application_command_autocomplete = 4
    modal_submit                     = 5


class InteractionResponseType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Interaction Callback Types </interactions/receiving-and-responding#interaction-response-object-interaction-callback-type>`
    """

    pong                                    = 1
    channel_message_with_source             = 4
    deferred_channel_message_with_source    = 5
    deferred_update_message                 = 6
    update_message                          = 7
    application_command_autocomplete_result = 8
    modal                                   = 9


class ApplicationFlags(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Application Flags </resources/application#application-object-application-flags>`
    """

    application_auto_moderation_rule_create_badge = 1 << 6
    gateway_presence                              = 1 << 12
    gateway_presence_limited                      = 1 << 13
    gateway_guild_members                         = 1 << 14
    gateway_guild_members_limited                 = 1 << 15
    verification_pending_guild_limit              = 1 << 16
    embedded                                      = 1 << 17
    gateway_message_content                       = 1 << 18
    gateway_message_content_limited               = 1 << 19
    application_command_badge                     = 1 << 23


class ApplicationRoleConnectionMetadataType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Application Role Connection Metadata Types </resources/application-role-connection-metadata#application-role-connection-metadata-object-application-role-connection-metadata-type>`
    """

    integer_less_than_or_equal     = 1
    integer_greater_than_or_equal  = 2
    integer_equal                  = 3
    integer_not_equal              = 4
    datetime_less_than_or_equal    = 5
    datetime_greater_than_or_equal = 6
    boolean_equal                  = 7
    boolean_not_equal              = 8


class AuditLogEvent(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Audit Log Events </resources/audit-log#audit-log-entry-object-audit-log-events>`
    """

    guild_update                                = 1
    channel_create                              = 10
    channel_update                              = 11
    channel_delete                              = 12
    channel_overwrite_create                    = 13
    channel_overwrite_update                    = 14
    channel_overwrite_delete                    = 15
    member_kick                                 = 20
    member_prune                                = 21
    member_ban_add                              = 22
    member_ban_remove                           = 23
    member_update                               = 24
    member_role_update                          = 25
    member_move                                 = 26
    member_disconnect                           = 27
    bot_add                                     = 28
    role_create                                 = 30
    role_update                                 = 31
    role_delete                                 = 32
    invite_create                               = 40
    invite_update                               = 41
    invite_delete                               = 42
    webhook_create                              = 50
    webhook_update                              = 51
    webhook_delete                              = 52
    emoji_create                                = 60
    emoji_update                                = 61
    emoji_delete                                = 62
    message_delete                              = 72
    message_bulk_delete                         = 73
    message_pin                                 = 74
    message_unpin                               = 75
    integration_create                          = 80
    integration_update                          = 81
    integration_delete                          = 82
    stage_instance_create                       = 83
    stage_instance_update                       = 84
    stage_instance_delete                       = 85
    sticker_create                              = 90
    sticker_update                              = 91
    sticker_delete                              = 92
    guild_scheduled_event_create                = 100
    guild_scheduled_event_update                = 101
    guild_scheduled_event_delete                = 102
    thread_create                               = 110
    thread_update                               = 111
    thread_delete                               = 112
    application_command_permission_update       = 121
    auto_moderation_rule_create                 = 140
    auto_moderation_rule_update                 = 141
    auto_moderation_rule_delete                 = 142
    auto_moderation_block_message               = 143
    auto_moderation_flag_to_channel             = 144
    auto_moderation_user_communication_disabled = 145


class OptionalAuditLogEntryInfoOverwrittenEntityType(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Optional Audit Entry Info </resources/audit-log#audit-log-entry-object-optional-audit-entry-info>`
    """

    role   = '0'
    member = '1'


class AutoModerationTriggerType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Trigger Types </resources/auto-moderation#auto-moderation-rule-object-trigger-types>`
    """

    keyword        = 1
    spam           = 3
    keyword_preset = 4
    mention_spam   = 5


class AutoModerationRuleKeywordPresetType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Keyword Preset Types </resources/auto-moderation#auto-moderation-rule-object-keyword-preset-types>`
    """

    profanity      = 1
    sexual_content = 2
    slurs          = 3


class AutoModerationRuleEventType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Event Types </auto-moderation#auto-moderation-rule-object-event-types>`
    """

    message_send = 1


class AutoModerationActionType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Action Types </resources/auto-moderation#auto-moderation-action-object-action-types>`
    """

    block_message      = 1
    send_alert_message = 2
    timeout            = 3


class ChannelVideoQualityMode(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Video Quality Modes </resources/channel#channel-object-video-quality-modes>`
    """

    auto = 1
    full = 2


class ChannelFlags(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Channel Flags </resources/channel#channel-object-channel-flags>`
    """

    pinned      = 1 << 1
    require_tag = 1 << 4


class ChannelSortOrderType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Sort Order Types </channel#channel-object-sort-order-types>`
    """

    latest_activity = 0
    creation_date   = 1


class ForumLayoutType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Forum Layout Types </resources/channel#channel-object-forum-layout-types>`
    """

    not_set      = 0
    list_view    = 1
    gallery_view = 2


class MessageType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Message Types </resources/channel#message-object-message-types>`
    """

    default                                      = 0
    recipient_add                                = 1
    recipient_remove                             = 2
    call                                         = 3
    channel_name_change                          = 4
    channel_icon_change                          = 5
    channel_pinned_message                       = 6
    user_join                                    = 7
    guild_boost                                  = 8
    guild_boost_tier_1                           = 9
    guild_boost_tier_2                           = 10
    guild_boost_tier_3                           = 11
    channel_follow_add                           = 12
    guild_discovery_disqualified                 = 14
    guild_discovery_requalified                  = 15
    guild_discovery_grace_period_initial_warning = 16
    guild_discovery_grace_period_final_warning   = 17
    thread_created                               = 18
    reply                                        = 19
    chat_input_command                           = 20
    thread_starter_message                       = 21
    guild_invite_reminder                        = 22
    context_menu_command                         = 23
    auto_moderation_action                       = 24
    role_subscription_purchase                   = 25
    interaction_premium_upsell                   = 26
    stage_start                                  = 27
    stage_end                                    = 28
    stage_speaker                                = 29
    stage_topic                                  = 31
    guild_application_premium_subscription       = 32


class MessageActivityType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Message Activity Types </resources/channel#message-object-message-activity-types>`
    """

    join         = 1
    spectate     = 2
    listen       = 3
    join_request = 5


class MessageFlags(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Message Flags </resources/channel#message-object-message-flags>`
    """

    crossposted                            = 1 << 0
    is_crosspost                           = 1 << 1
    suppress_embeds                        = 1 << 2
    source_message_deleted                 = 1 << 3
    urgent                                 = 1 << 4
    has_thread                             = 1 << 5
    ephemeral                              = 1 << 6
    loading                                = 1 << 7
    failed_to_mention_some_roles_in_thread = 1 << 8
    suppress_notifications                 = 1 << 12
    is_voice_message                       = 1 << 13



class OverwriteType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Overwrite Structure </resources/channel#overwrite-object-overwrite-structure>`
    """

    role   = 0
    member = 1


class EmbedType(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Embed Types </resources/channel#embed-object-embed-types>`
    """

    rich    = 'rich'
    image   = 'image'
    video   = 'video'
    gifv    = 'gifv'
    article = 'article'
    link    = 'link'


class AllowedMentionsType(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Allowed Mention Types </resources/channel#allowed-mentions-object-allowed-mention-types>`
    """

    roles    = 'roles'
    users    = 'users'
    everyone = 'everyone'


class GuildVerificationLevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Verification Level </resources/guild#guild-object-verification-level>`
    """

    none      = 0
    low       = 1
    medium    = 2
    high      = 3
    very_high = 4


class GuildDefaultMessageNotificationLevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Default Message Notification Level </resources/guild#guild-object-default-message-notification-level>`
    """

    all_messages  = 0
    only_mentions = 1
    

class GuildExplicitContentFilterLevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Explicit Content Filter Level </resources/guild#guild-object-explicit-content-filter-level>`
    """

    disabled              = 0
    members_without_roles = 1
    all_members           = 2


class GuildFeature(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Guild Features </resources/guild#guild-object-guild-features>`
    """

    animated_banner                           = 'ANIMATED_BANNER'
    animated_icon                             = 'ANIMATED_ICON'
    application_command_permissions_v2        = 'APPLICATION_COMMAND_PERMISSIONS_V2'
    auto_moderation                           = 'AUTO_MODERATION'
    banner                                    = 'BANNER'
    community                                 = 'COMMUNITY'
    creator_monetizable_provisional           = 'CREATOR_MONETIZABLE_PROVISIONAL'
    creator_store_page                        = 'CREATOR_STORE_PAGE'
    developer_support_server                  = 'DEVELOPER_SUPPORT_SERVER'
    discoverable                              = 'DISCOVERABLE'
    featurable                                = 'FEATURABLE'
    invites_disabled                          = 'INVITES_DISABLED'
    invite_splash                             = 'INVITE_SPLASH'
    member_verification_gate_enabled          = 'MEMBER_VERIFICATION_GATE_ENABLED'
    more_stickers                             = 'MORE_STICKERS'
    news                                      = 'NEWS'
    partnered                                 = 'PARTNERED'
    preview_enabled                           = 'PREVIEW_ENABLED'
    role_icons                                = 'ROLE_ICONS'
    role_subscriptions_available_for_purchase = 'ROLE_SUBSCRIPTIONS_AVAILABLE_FOR_PURCHASE'
    role_subscriptions_enabled                = 'ROLE_SUBSCRIPTIONS_ENABLED'
    ticketed_events_enabled                   = 'TICKETED_EVENTS_ENABLED'
    vanity_url                                = 'VANITY_URL'
    verified                                  = 'VERIFIED'
    vip_regions                               = 'VIP_REGIONS'
    welcome_screen_enabled                    = 'WELCOME_SCREEN_ENABLED'
    raid_alerts_disabled                      = 'RAID_ALERTS_DISABLED'
    # undocumented
    text_in_voice_enabled                     = 'TEXT_IN_VOICE_ENABLED'


class GuildMFALevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Mfa Level </resources/guild#guild-object-mfa-level>`
    """

    none     = 0
    elevated = 1


class GuildNSFWLevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Guild Nsfw Level </resources/guild#guild-object-guild-nsfw-level>`
    """

    default        = 0
    explicit       = 1
    safe           = 2
    age_restricted = 3


class GuildSystemChannelFlags(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`System Channel Flags </resources/guild#guild-object-system-channel-flags>`
    """

    suppress_join_notifications                              = 1 << 0
    suppress_premium_subscriptions                           = 1 << 1
    suppress_guild_reminder_notifications                    = 1 << 2
    suppress_join_notification_replies                       = 1 << 3
    suppress_role_subscription_purchase_notifications        = 1 << 4
    suppress_role_subscription_purchase_notification_replies = 1 << 5


class GuildPremiumTier(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Premium Tier </resources/guild#guild-object-premium-tier>`
    """

    none   = 0
    tier_1 = 1
    tier_2 = 2
    tier_3 = 3


class GuildMemberFlags(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Guild Member Flags </resources/guild#guild-member-object-guild-member-flags>`
    """

    did_rejoin            = 1 << 0
    completed_onboarding  = 1 << 1
    bypasses_verification = 1 << 2
    started_onboarding    = 1 << 3


class IntegrationExpireBehaviorType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Integration Expire Behaviors </resources/guild#integration-object-integration-expire-behaviors>`
    """

    remove_role = 0
    kick        = 1


class GuildOnboardingPromptType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Prompt Types </resources/guild#guild-onboarding-object-prompt-types>`
    """

    multiple_choice = 0
    dropdown        = 1


class GuildScheduledEventPrivacyLevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Guild Scheduled Event Privacy Level </resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-privacy-level>`
    """

    guild_only = 2


class GuildScheduledEventEntityType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Guild Scheduled Event Entity Types </resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-entity-types>`
    """

    stage_instance = 1
    voice          = 2
    external       = 3


class GuildScheduledEventStatus(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Guild Scheduled Event Status </resources/guild-scheduled-event#guild-scheduled-event-object-guild-scheduled-event-status>`
    """

    scheduled = 1
    active    = 2
    completed = 3
    canceled  = 4


class InviteTargetType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Invite Target Types </resources/invite#invite-object-invite-target-types>`
    """

    stream               = 1
    embedded_application = 2


class StageInstancePrivacyLevel(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Privacy Level </resources/stage-instance#stage-instance-object-privacy-level>`
    """

    public     = 1
    guild_only = 2


class StickerType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Sticker Types </resources/sticker#sticker-object-sticker-types>`
    """

    standard = 1
    guild    = 2


class StickerFormatType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Sticker Format Types </resources/sticker#sticker-object-sticker-format-types>`
    """

    png    = 1
    apng   = 2
    lottie = 3
    gif    = 4


class UserFlags(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`User Flags </resources/user#user-object-user-flags>`
    """

    staff                    = 1 << 0
    partner                  = 1 << 1
    hypesquad                = 1 << 2
    bug_hunter_level_1       = 1 << 3
    hypesquad_online_house_1 = 1 << 6
    hypesquad_online_house_2 = 1 << 7
    hypesquad_online_house_3 = 1 << 8
    premium_early_supporter  = 1 << 9
    team_pseudo_user         = 1 << 10
    bug_hunter_level_2       = 1 << 14
    verified_bot             = 1 << 16
    verified_developer       = 1 << 17
    certified_moderator      = 1 << 18
    bot_http_interactions    = 1 << 19
    active_developer         = 1 << 22


class UserPremiumType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Premium Types </resources/user#user-object-premium-types>`
    """

    none          = 0
    nitro_classic = 1
    nitro         = 2
    nitro_basic   = 3


class ConnectionVisibilityType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Visibility Types </resources/user#connection-object-visibility-types>`
    """

    none     = 0
    everyone = 1


class WebhookType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Webhook Types </resources/webhook#webhook-object-webhook-types>`
    """

    incoming         = 1
    channel_follower = 2
    application      = 3


class ActivityType(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Activity Types </topics/gateway-events#activity-object-activity-types>`
    """

    game      = 0
    streaming = 1
    listening = 2
    watching  = 3
    custom    = 4
    competing = 5


class ActivityFlags(enum.IntFlag, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Activity Types </topics/gateway-events#activity-object-activity-types>`
    """

    instance                    = 1 << 0
    join                        = 1 << 1
    spectate                    = 1 << 2
    join_request                = 1 << 3
    sync                        = 1 << 4
    play                        = 1 << 5
    party_privacy_friends       = 1 << 6
    party_privacy_voice_channel = 1 << 7
    embedded                    = 1 << 8


class TeamMemberMembershipState(enum.IntEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Data Models Membership State Enum </topics/teams#data-models-membership-state-enum>`
    """

    invited  = 1
    accepted = 2


class WidgetStyleOption(enum.StrEnum, metaclass = EnumMeta):

    """
    |dsrc| 
    :ddoc:`Widget Style Options </resources/guild#get-guild-widget-image-widget-style-options>`
    """

    shield	= 'shield' 
    banner1	= 'banner1' 
    banner2	= 'banner2' 
    banner3	= 'banner3' 
    banner4	= 'banner4' 


class SpeechFlags(enum.IntFlag):

    """
    |dsrc|
    :ddoc:`Speaking Flags </topics/voice-connections#speaking>`
    """

    microphone = 1
    soundshare = 2
    priority   = 4


class TimestampStyle(enum.StrEnum):

    """
    |dsrc|
    :ddoc:`Message Formatting Timestamp Styles </reference#message-formatting-timestamp-styles>`
    """

    short_time      = 't'
    long_time       = 'T'
    short_date      = 'd'
    long_date       = 'D'
    short_date_time = 'f'
    long_date_time  = 'F'
    relative_time   = 'R'